@@ -43,7 +43,6 @@ class HeareClient(object):
                 auth_str += password
             self._auth_header = "Authorization: Basic %s " % str(base64.b64encode(bytes(auth_str, 'utf-8')), 'utf-8')
 
-        self.client = self._create_new_connection()
         self.pool = ThreadPool(10)
         self.serializer = Serializer()
         self.deserializer = Deserializer()
@@ -65,11 +64,11 @@ class HeareClient(object):
         self.emitter.emit("open")
         self.reconnect_counter = 1
 
-    def _on_close(self):
+    def _on_close(self, *args):
         self.emitter.emit("close")
 
-    def _on_error(self, error, _):
-        logger.error(error)
+    def _on_error(self, socket, error):
+        logger.error(repr(error))
         try:
             self.emitter.emit('error', error)
             self.client.close()
@@ -80,8 +79,6 @@ class HeareClient(object):
             "Disconnecting on error, reconnecting in %d seconds." % sleep_time)
         self.reconnect_counter = min(self.reconnect_counter * 2, 60)
         time.sleep(sleep_time)
-        self.client = self._create_new_connection()
-        self.run_forever()
 
     def _on_message(self, _, message):
         try:
@@ -109,13 +106,21 @@ class HeareClient(object):
         self.emitter.remove_listener(event_name, func)
 
     def run_forever(self, ping_interval=1):
-        try:
-            logger.info("Connecting...")
-            self.client.run_forever(ping_interval=ping_interval)
-        except Exception as _:
-            logger.exception("Error connecting to server")
-        finally:
-            logger.info("Client run_forever complete.")
+        while True:
+            try:
+                logger.info("Connecting...")
+                self.client = self._create_new_connection()
+                self.client.run_forever(ping_interval=ping_interval)
+            except KeyboardInterrupt as _:
+                logger.exception("Exiting")
+                self.client.close()
+                break
+            except Exception as _:
+                logger.exception("Error connecting to server")
+                if self.client:
+                    self.client.close()
+
+        logger.info("Client run_forever complete.")
 
     def close(self):
         self.client.close()
